@@ -28,7 +28,10 @@ test_that("random.dag runs as expected", {
 })
 
 test_that("random.dag always produces a DAG", {
-    m <- random.dag(5,5)
-    m[m!=0] <- 1
-    expect_warning(igraph::topo_sort(igraph::graph.adjacency(m)), NA)
+    ### This test requires the igraph pkg
+    if(requireNamespace("igraph", quietly = TRUE)){
+        m <- random.dag(5,5)
+        m[m!=0] <- 1
+        expect_warning(igraph::topo_sort(igraph::graph.adjacency(m)), NA)
+    }
 })
